pub struct ExtraRunningState {
pub(super) t_ss: SharedSecret,
pub(super) auths_ss: SharedSecret,
pub(super) attr_signing_key: HS256,
pub(super) t_sealing_secret: SealingKey,
pub(super) auths_sealing_secret: SealingKey,
pub(super) master_enc_key: Option<PublicKey>,
}Fields
t_ss: SharedSecretHybrid post-quantum shared secret with the transcryptor.
auths_ss: SharedSecretHybrid post-quantum shared secret with the authentication server.
attr_signing_key: HS256Key used to sign Attrs, shared with the authentication server
t_sealing_secret: SealingKeyKey used to (un)seal messages to and from the transcryptor
auths_sealing_secret: SealingKeyKey used to (un)seal messages to and from the authentication server
master_enc_key: Option<PublicKey>The master encryption key x_T x_PHC B, derived from the transcryptor’s sealed master key
part. None until PHC has been able to unseal it (e.g. while the transcryptor has not yet
published a sealed part). Held here because it is no longer part of the (public)
constellation.